unit ufrmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, XPMan, Menus;

type
  TfrmMain = class(TForm)
    StatusBar1: TStatusBar;
    lsbImages: TListBox;
    XPManifest1: TXPManifest;
    ScrollBox1: TScrollBox;
    imgPreview: TImage;
    mnuMain: TMainMenu;
    mnuArchivo: TMenuItem;
    mnuSalir: TMenuItem;
    mnuNueva: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    mnuGuardarImagen: TMenuItem;
    procedure mnuSalirClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure mnuNuevaClick(Sender: TObject);
    procedure lsbImagesClick(Sender: TObject);
    procedure mnuGuardarImagenClick(Sender: TObject);
  private
    { Private declarations }
    Filename: AnsiString;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses ufrmScan, JPEG;

procedure CopyJpegFrom(Src, Dst: TStream);
var
  Buffer: PByteArray;
  Size: Integer;
begin
  GetMem(Buffer,$10000);
  try
    while TRUE do
    begin
      FillChar(Buffer^,4,#0);
      Src.Read(Buffer^,4);
      Size:= ((Buffer[2] shl 8) + Buffer[3]) - 2;
      case Buffer[1] of
        $01,$D0..$D8: begin
                        Dst.WriteBuffer(Buffer^,2);
                        Src.Seek(-2,soFromCurrent);
                      end;
        $D9: begin
               Dst.WriteBuffer(Buffer^,2);
               // Salimos del procedure
               Exit;
             end;
        $DA: begin
               Dst.WriteBuffer(Buffer^,2);
               Src.Seek(-2,soFromCurrent);
               while TRUE do
               begin
                 Src.ReadBuffer(Buffer^,1);
                 if Buffer[0] = $FF then
                 begin
                   Src.ReadBuffer(Buffer^,1);
                  if (Buffer[0] <> 0) and not (Buffer[0] in [$D0..$D7]) then
                   begin
                     Src.Seek(-2,soFromCurrent);
                     break;
                   end else
                   begin
                     Buffer[1]:= Buffer[0];
                     Buffer[0]:= $FF;
                     Dst.WriteBuffer(Buffer^,2);
                   end;
                 end else
                   Dst.WriteBuffer(Buffer^,1);
               end;
             end
        else
        begin
          Dst.WriteBuffer(Buffer^,4);
          Src.ReadBuffer(Buffer^,Size);
          Dst.WriteBuffer(Buffer^,Size);
        end;
      end;
    end;
  finally
    FreeMem(Buffer);
  end;
end;


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  mnuNueva.Click;
end;

const
  BLOCKSIZE = 512;

procedure TfrmMain.lsbImagesClick(Sender: TObject);
var
  BufferSize: Integer;
  Offset: int64;
  HighOffset: Cardinal;
  Stream: TFileStream;
  Src,Dst: TMemoryStream;
  Jpg: TJPEGImage;
begin
  // Borramos la imagen anterior
  imgPreview.Picture:= nil;
  if lsbImages.ItemIndex < 0 then
    Exit;
  if Self.Filename = EmptyStr then
    Exit;
  // Obntenemos la posicion de la imagen dentro del fichero o disco
  if not TryStrToInt64('$'+lsbImages.Items[lsbImages.ItemIndex],Offset) then
    Exit;
  try
    // Abrimos el fichero o disco
    Stream:= TFileStream.Create(Self.Filename,fmOpenRead or fmShareDenyNone);
    try
      // Separamos los 32 bits mas altos
      HighOffset:= Offset shr 32;
      // Vamos a la posicion donde se encuentra la imagen
      if (SetFilePointer(Stream.Handle,Offset and $FFFFFFFF,@HighOffset,FILE_BEGIN) = DWORD(-1)) and
        (GetLastError <> NO_ERROR ) then
        Exit;
      // Creamos los strems temporales
      Src:= TMemoryStream.Create;
      Dst:= TMemoryStream.Create;
      try
        // Comenzamos con 1 MB
        BufferSize:= 1*1024*1024;
        // Ajustamos el tamao del buffer al tamao del bloque
        BufferSize:= BufferSize - (BufferSize mod BLOCKSIZE);
        while True do
        begin
          Dst.Clear;
          Src.Seek(0,soFromEnd);
          // Leemos los datos
          Src.CopyFrom(Stream,BufferSize);
          Src.Position:= 0;
          try
            // Intentamos recuperar la foto
            CopyJpegFrom(Src, Dst);
          except
            // Si nos faltan datos
            On E: EReadError do
              // Y el tamao es menor de 8 MB
              // (Se puede poner un tamao mayor si es necesario)
              if Src.Size < 8*1024*1024 then
                // Leemos un poco mas
                Continue
              else
                raise EReadError.Create(E.Message);
          end;
          break;
        end;
        Dst.Position:= 0;
        Jpg:= TJPEGImage.Create;
        try
          try
            // Cargamos la imagen
            Jpg.LoadFromStream(Dst);
            // Y la colocamos en pantalla
            imgPreview.Picture.Assign(jpg);
          except
            imgPreview.Picture:= nil;
          end;
        finally
          jpg.Free;
        end;
      finally
        Src.Free;
        Dst.Free;
      end;
    finally
      Stream.Free;
    end;
  except
    //
  end;
end;

procedure TfrmMain.mnuGuardarImagenClick(Sender: TObject);
begin
  if lsbImages.ItemIndex < 0 then
    Exit;
  if imgPreview.Picture.Graphic = nil then
    Exit;
  with TSaveDialog.Create(nil) do
  try
    Filter:= '*.jpg|*.jpg|*.*|*.*';
    Filename:= lsbImages.Items[lsbImages.ItemIndex] + '.jpg';
    if Execute then
      imgPreview.Picture.SaveToFile(Filename);
  finally
    Free;
  end;
end;

procedure TfrmMain.mnuNuevaClick(Sender: TObject);
begin
  // Preguntamos si quiere borrar los resultado anteriores
  if (lsbImages.Items.Count = 0) or (MessageBox(Handle,
    'Para iniciar una bsqueda es necesario borrar los resultados anteriores',
    'Desea continuar?',
    MB_YESNO or MB_SETFOREGROUND or MB_TASKMODAL) = ID_YES)   then
    with TfrmScan.Create(nil) do
    try
      lsbImages.Clear;
      ShowModal;
      Self.Filename:= Filename;
    finally
      Free;
    end;
end;

procedure TfrmMain.mnuSalirClick(Sender: TObject);
begin
  Close;
end;

end.
