unit ufrmScan;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls;

type
  TfrmScan = class(TForm)
    cmbDisk: TComboBox;
    Label1: TLabel;
    btnBuscar: TButton;
    lbStatus: TLabel;
    btnCancelar: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnBuscarClick(Sender: TObject);
    procedure btnCancelarClick(Sender: TObject);
  private
    { Private declarations }
    procedure ScanFile;
  public
    { Public declarations }
    Filename: AnsiString;
  end;

var
  frmScan: TfrmScan;

implementation

{$R *.dfm}

uses ufrmMain;

function FormatFileSize(const Bytes: int64): String;
const
  B = 1;          // Byte
  KB = 1024 * B;  // Kilobyte
  MB = 1024 * KB; // Megabyte
  GB = 1024 * MB; // Gigabyte
begin
  if Bytes > GB then
    Result := FormatFloat('0.00 GB', Bytes / GB)
  else if Bytes > MB then
    Result := FormatFloat('0.00 MB', Bytes / MB)
  else if Bytes > KB then
    Result := FormatFloat('0.00 KB', Bytes / KB)
  else
    Result := FormatFloat('0.00 Bytes', Bytes);
end;

function Status(const Bytes: int64; const Img: Integer): String;
begin
  Result:= Format('%s  / %d Imagenes',[FormatFileSize(Bytes),Img]);
end;

procedure TfrmScan.btnBuscarClick(Sender: TObject);
begin
  if cmbDisk.ItemIndex < 0 then
    Exit;
  Self.Filename:= EmptyStr;
  // Si escogemos "Usar un archivo" mostramos un TOpenDialog
  if cmbDisk.ItemIndex = 0 then
    with TOpenDialog.Create(nil) do
    try
      if Execute then
        Self.Filename:= Filename
      else
        Exit;
    finally
      Free;
    end
  else
    Self.Filename:= cmbDisk.Items[cmbDisk.ItemIndex];
  // Cambiamos el estado de los botones
  cmbDisk.Enabled:= FALSE;
  btnBuscar.Enabled:= FALSE;
  btnCancelar.Enabled:= TRUE;
  try
    try
      Scanfile;
    except
      // 
    end;
  finally
    // Nos aseguramos de que vamos a poder cerrar la ventana
    btnCancelar.Enabled:= FALSE;
    Close;
  end;
end;

procedure TfrmScan.btnCancelarClick(Sender: TObject);
begin
  btnCancelar.Enabled:= FALSE;
end;

procedure TfrmScan.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:= not btnCancelar.Enabled;  
end;

procedure TfrmScan.FormCreate(Sender: TObject);
var
  i: Integer;
  C: AnsiChar;
  H: Integer;
  Str: AnsiString;
begin
  // Ponemos todo a cero
  Self.Filename:= EmptyStr;
  lbStatus.Caption:= Status(0,0);
  cmbDisk.Clear;
  cmbDisk.Items.Add('Usar un archivo ...');
  // Buscamos todas las unidades
  for C:= 'A' to 'Z' do
  begin
    Str:= '\\.\' + C + ':';
    H:= FileOpen(PAnsiChar(Str),fmOpenRead or fmShareDenyNone);
    // Si se puede abrir lo aadimos a la lista
    if H > 0 then
    begin
      CloseHandle(H);
      cmbDisk.Items.Add(Str);
    end;
  end;
  // Buscamos todos los discos (al menos los 16 primeros)
  for i:= 0 to 16 do
  begin
    Str:= '\\.\PhysicalDrive' + IntTostr(i);
    H:= FileOpen(PAnsiChar(Str),fmOpenRead or fmShareDenyNone);
    // Si se puede abrir lo aadimos a la lista
    if H > 0 then
    begin
      CloseHandle(H);
      cmbDisk.Items.Add(Str);
    end;
  end;
end;

const
  BLOCKSIZE = 512;

procedure TfrmScan.ScanFile;
var
  BufferSize: Integer;
  i, l, Count: Integer;
  Offset: int64;
  Buffer, P: PAnsiChar;
  Stream: TFileStream;
begin
  // Usamos un buffer de 32 MB
  BufferSize:= 32 * 1024 * 1024;
  // Ajustamos el tamao del buffer al tamao del bloque
  BufferSize:= BufferSize - (BufferSize mod BLOCKSIZE);
  // Abrimos el fichero o disco
  Stream:= TFileStream.Create(Self.Filename,fmOpenRead or fmShareDenyNone);
  try
    Count:= 0;
    // Reservamos espacio para el buffer
    GetMem(Buffer,BufferSize);
    try
      Offset:= 0;
      // Leemos el fichero en bloques de 32 MB
      l:= Stream.Read(Buffer^,BufferSize);
      while l > 0 do
      begin
        lbStatus.Caption:= Status(Offset,Count);
        Application.ProcessMessages;
        // Usamos el boton cancelar para finalizar el proceso
        if not btnCancelar.Enabled then
          Exit;
        P:= Buffer;
        // Recorremos el buffer bloque a bloque
        for i:= 1 to l div BLOCKSIZE do
        begin
          // Si encontramos el comienzo de una imagen JPEG
          if CompareMem(P,PAnsiChar(#$FF#$D8#$FF),3) then
          begin
            // Aadimos su posicin a la lista
            frmMain.lsbImages.Items.Add(IntToHex(Offset,16));
            // Incrementamos la cuenta de imagenes encontradas
            inc(Count);
          end;
          inc(P,BLOCKSIZE);
          inc(Offset,BLOCKSIZE);
        end;
        l:= Stream.Read(Buffer^,BufferSize);
      end;
    finally
      FreeMem(Buffer);
    end;
  finally
    Stream.Free;
  end;
end;

end.
